/* geomviewer.h: main S2PLOT code
 *
 * ==================================================================
 * S2PLOT is freely available for non-commercial and educational use,
 * however, it is not public-domain software. The programming library
 * and documentation may not be redistributed or sub-licensed in any
 * form without permission from Swinburne University of
 * Technology. The programming library and sample codes are provided
 * without warranty.
 *
 * We would appreciate it if research outcomes using S2PLOT would
 * provide the following acknowledgement:
 *
 * "Three-dimensional visualisation was conducted with the S2PLOT
 * progamming library"
 *
 * and a reference to 
 *
 * D.G.Barnes, C.J.Fluke, P.D.Bourke & O.T.Parry, 2006, Publications
 * of the Astronomical Society of Australia, 23(2), 82-93.
 *
 * S2PLOT development was funded through the Researcher Developer
 * Grant Scheme at Swinburne University of Technology.
 * ==================================================================
 *
 * $Id: geomviewer.h 5279 2008-09-04 05:31:19Z dbarnes $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

// this MUST be before the s2glut.h include as it sets a define 
// that is examined when glext.h is implicitly included by gl.h.
#if defined(S2LINUX)
#define GL_GLEXT_PROTOTYPES
#endif

//#include "s2glut.h"
#include "s2opengl.h"
#include "s2win.h"

#include <sys/time.h>
#include <sys/resource.h>
#include <sys/file.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/times.h>

//#ifdef __APPLE__
//#include "hiddenMouseCursor.h"
//#endif

#include "s2base.h"
#include "s2const.h"
#include "paulslib.h"
#include "bitmaplib.h"
#include "opengllib.h"

// Warp map types
#define POLAR 1
#define RECTANGULAR 2

// One triangle 
typedef struct {
   double u[3],v[3]; // Texture coordinates
   XYZ p[3];         // Vertices 
   XYZ wp[3];        // Vertices for warpmap
   double wi[3];     // Intensity of node, for warpmap
} DOMEFACE;

typedef struct {
   double x,y;         // Coordinate
   double u,v;         // UV coordinate
   double i;           // Intensity
} MESHNODE;

#if (0)
// - moved to s2base.h
typedef struct {
  XYZ vp;           	/* View position            */
  XYZ vd;           	/* View direction vector    */
  XYZ vu;           	/* View up direction        */
  XYZ pr;		/* Point to rotate about    */
  double focallength;   /* Focal Length along vd    */
  XYZ focus;            /* Focal point - Derived    */
  double aperture;  	/* Camera VERTICAL aperture */
  double eyesep;	/* Eye separation	    */
  double speed;         /* Speed in vd direction    */
  double fishrotate;    /* Rotate fisheye camera    */
} CAMERA;
#endif

typedef struct {
	int debug;
	int stereo;
	int fullscreen;
   int screenwidth;
   int screenheight;
	int dometype;
  char meshfile[128];
	int showdomeintensity;
	int rendermode;
	int showtextures;
	int showboundingbox;
	int recordimages;
	int windowdump;
	int makeqtvr;
	int showhelpinfo;
	int projectiontype;
	int selectmode;
	int topbottom;
	double targetfps;
	int doantialias;
	int cursortype;
	int interaction;
	int autopilot;
	XYZ autospin;
	XYZ scalefactor;
	int combineeyes;
	double interactspeed;
	double deltamove;
	COLOUR background;
	int sphereresolution;
	double pointscale,linescale;
   int faceorder;
   int twosided;
   int localviewer;
	int texturetype;
	CAMERA camerahome;
} OPTIONS;

typedef struct {
	int currentbutton;
	int mousexlast,mouseylast;
  int spaceballxlast, spaceballylast, spaceballzlast;
	double framerate;
} INTERFACESTATE;

#if defined(BUILDING_S2PLOT)
#define MAXVRMLLEN 32
#endif
typedef struct {
	XYZ p;
	COLOUR colour;
	double size;
#if defined(BUILDING_S2PLOT)
  char whichscreen[10];
  char VRMLname[MAXVRMLLEN];
#endif
} DOT;

#if defined(BUILDING_S2PLOT)
typedef struct {
  XYZ p;
  COLOUR colour;
  double size;
  int trans;
  double alpha;
  char whichscreen[10];
} TRDOT; /* transparent dot */
#endif

typedef struct {
  XYZ p[2];
  COLOUR colour[2];
  double width;
#if defined(BUILDING_S2PLOT)
  char whichscreen[10];
  char VRMLname[MAXVRMLLEN];
  int stipple_factor; // 0 means no stipple
  unsigned short stipple_pattern;
#endif
} LINE;

typedef struct {
  XYZ p[3];
  XYZ n[3];
  COLOUR colour[3];
#if defined(BUILDING_S2PLOT)
  char whichscreen[10];
  char VRMLname[MAXVRMLLEN];
#endif
} FACE3;

typedef struct {
  XYZ p[4];
  XYZ n[4];
  COLOUR colour[4];
#if defined(BUILDING_S2PLOT)
  char whichscreen[10];
  char VRMLname[MAXVRMLLEN];
#endif
} FACE4;

typedef struct {
  XYZ p[4];
  COLOUR colour;
  char texturename[64];
  int width,height;
  double scale;
  int trans;
  BITMAP4 *rgba;
  GLuint textureid;
#if defined(BUILDING_S2PLOT)
  double alpha; /* transparency: 1.0 = default = opaque */
  char whichscreen[10]; 
  char VRMLname[MAXVRMLLEN];
#endif
} FACE4T;

typedef struct {
  XYZ p;
  double r;
  COLOUR colour;
#if defined(BUILDING_S2PLOT)
  char whichscreen[10];
  char VRMLname[MAXVRMLLEN];
#endif
} BALL;

typedef struct {
  XYZ p;
  double r;
  COLOUR colour;
  char texturename[64];
  int width,height;
  BITMAP4 *rgba;
  GLuint textureid;
#if defined(BUILDING_S2PLOT)
  float texture_phase; // phase of texture [longitude in 0->1]
  XYZ axis; // axis of rotation
  float rotation; // angle of rotation in degrees
  char whichscreen[10];
#endif
} BALLT;

typedef struct {
  XYZ p;
  XYZ n;
  double r1,r2;
  COLOUR colour;
#if defined(BUILDING_S2PLOT)
  char whichscreen[10];
#endif
} DISK;

typedef struct {
  XYZ p1,p2;
  double r1,r2;
  COLOUR colour;
#if defined(BUILDING_S2PLOT)
  char whichscreen[10];
  char VRMLname[MAXVRMLLEN];
#endif
} CONE;

typedef struct {
  int nv;
  int p[4];
} OFFFACE;

#define MAXLABELLEN 256
typedef struct {
  XYZ p;
  XYZ right,up;
  COLOUR colour;
  char s[MAXLABELLEN];
#if defined(BUILDING_S2PLOT)
  char whichscreen[10];
  char VRMLname[MAXVRMLLEN];
#endif
} LABEL;

typedef struct {
  GLfloat p[4];
  GLfloat c[4];
} ALIGHT;

#define SHUTTERGLASSES 0
#define ANAGLYPH_RB    1
#define ANAGLYPH_RG    2

#define INSPECT 0
#define FLY     1
#define WALK    2

#define NOCURSOR    0
#define ARROWCURSOR 1
#define CROSSCURSOR 2

#define HOME        1
#define FOCUS       2
#define FRONT       3
#define BACK        4
#define LEFT        5
#define RIGHT       6
#define TOP         7
#define BOTTOM      8
#define AXONOMETRIC 9

#define IMAGESPLIT 8

#define KEYBOARDCONTROL 0
#define MOUSECONTROL    1

// RO access to pmin, pmax for device drivers
XYZ _s2priv_pmin(void);
XYZ _s2priv_pmax(void);

// internal error reporting
void _s2debug(char *fn, char *messg, ...);
void _s2warn(char *fn, char *messg, ...);
void _s2error(char *fn, char *messg, ...);
void _s2warnk(char key, char *messg, ...);

/* Stereo.c */
void HandleDisplay(void);

void CreateOpenGL(void);
void MakeGeometry(int, int);
void MakeLighting(void);
void MakeMaterial(void);
void HandleReshape(int,int);
void HandleKeyboard(unsigned char key,int x, int y);
void HandleSpecialKeyboard(int key,int x, int y);
void RotateCamera(double,double,double,int);
void FlyCamera(double);
void TranslateCamera(double,double,int);
void HandleMouse(int,int,int,int);
void HandleMouseMotion(int,int);
void HandlePassiveMotion(int,int);

void HandleSpaceballButton(int, int);
void HandleSpaceballMotion(int, int, int);
void HandleSpaceballRotate(int, int, int);


void HandleVisibility(int vis);
void HandleIdle(void);
int  ReadGeometry(char *);
void CleanGeometry(void);
#if defined(BUILDING_S2PLOT)
void SaveGeomFile(char *);
#else
void SaveGeomFile(void);
#endif
int  ReadOFF(char *);
void DeleteGeometry(void);
void CameraHome(int);
int  AutoPilot(int,char *);


void ReadViewFile(char *);
#if !defined(BUILDING_VIEWER)
void SaveViewFile(CAMERA, char *);
#else
void SaveViewFile(CAMERA);
#endif
void AppendCameraPosition(void);
void CreateAxes(void);
void DrawExtras(void);
void CreateProjection(int);
void AddLine2Database(XYZ,XYZ,COLOUR,COLOUR,double);
void AddFace2Database(XYZ *,int,COLOUR,double,XYZ);
void AddMarker2Database(int,double,XYZ,COLOUR);
void CalcBounds(void);
void UpdateBounds(XYZ);
void CleanExit(void);

#if (0)
// menus.c
void HandleMainMenu(int);
void HandleAnimMenu(int);
void HandleWindowMenu(int);
void HandleRenderMenu(int);
void HandleSpeedMenu(int);
void HandleCameraMenu(int);
void HandleStereoMenu(int);
void HandleAppearMenu(int);
void HandleFileMenu(int);
void HandleViewMenu(int);
void HandleProjectionMenu(int);

// dome.c
void DomeTextures(void);
int  DomeCreate(void);
void DrawDome(int,int);
void DomeSplitFace(DOMEFACE *,int *);
void DomeFlatten(XYZ *,double);
int  ReadMesh(char *);
void MakeDefaultMesh(void);
void CalcWarpData(void);
void EstimateWarp(double,double,double *,double *,double *);
void SetVertex(DOMEFACE);
int MirrorPosition(XYZ,double,XYZ,XYZ,XYZ *);
#endif

// misc.c
void CreateASphere(XYZ,double,int,int,int);
void CreateAPlanet(XYZ,double,int,int,int,float,XYZ,float);
void GiveUsage(char *);
int  ReadVector(FILE *,XYZ *);
int  ReadColour(FILE *,COLOUR *);


